%%
%   IBM 4758 CCA API
%
%   Key Import-Export Attack (Pin-Derivation-Key is obtain in clear
%   without key conjuring).
%
%   The attacker has acces to KPI-last, Key-Import, and Key-Export
%   commands.
%%

    fun enc/2.      % symmetric encryption
    fun penc/2.     % public key encryption

    % types of keys:
    % fun dat/0.    % data keys  (data = 0)
    fun imp/0.      % import keys
    fun exp/0.      % export keys
    fun pin/0.      % key derivation keys
    fun kp/0.       % partial key type

    fun kp12/0.     % a partial key
    fun kp3/0.      % a partial key - the part of the intruder

    fun km/0.       % the security module's master key 

    fun c/0.        % some value known by the intruder

    fun pdkey/0.    % a Pin Derivation Key

    fun sec/0.

%%%%%%%%%%%% Parameters for ProVerif

    proverif 'param selFun = Nounifset'.

%%%%%%%%%%%% Auxiliary Prolog predicates

    type(0).   % data=0
    type(imp).
    type(exp).
    type(pin).
    
%%%%%%%%%%%% THE INITIAL KNOWLEDGE %%%%%%%%

    rule c.
    % rule dat.  % dat=0
    rule imp.
    rule exp.
    rule pin.
    rule kp.

    rule kp3.
    rule enc(pdkey, kp12+kp3+pin). % = {pdkey}_{kek+pin}, because kek=kp12+kp3
                                  

%%%%%%%%%%%% INTRUDER RULES %%%%%%%%%%%%%%%

    rule  X, K -> enc(X,K).
    rule  enc(X,K), K -> X.

%%%%%%%%%%%% PROTOCOL RULES %%%%%%%%%%%%%%%

%%% Encipher
    rule  X, enc(Key, km+0)  ->  enc(X, Key).

%%% Decipher
    rule  enc(X, Key), enc(Key, km+0)  ->  X.

%%% Key Import
    rule  enc(Key, KEK+Type), Type, enc(KEK, km+imp)  ->  enc(Key, km+Type)
    :- type(Type).

%%% Key Export
    rule  enc(Key, km+Type), Type, enc(KEK, km+exp)  ->  enc(Key, KEK+Type)
    :- type(Type).


%%% Key Part Import

    % KPI-First + KPI-Add/Middle
    rule  enc( kp12, km+kp+imp ).
    
    % KPI-Last:   X, Type, enc(Y, km+kp+Type)  ->  enc(X+Y, km+Type).
    % unfolded:
    
    rule  X, Type, km+kp+Type  ->  enc(X, km+Type)
    :- type(Type).

    rule  X, imp  ->  enc(X+kp12, km+imp).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

query pdkey.
