%%
%  IBM 4758  CCA API
%
%  Recomendation 2, Role E:
%
%  The intuder has acces to Encipher, Decipher, KeyImport, KeyExport,
%  KeyTranslate (which is subsumed by KeyImport and KeyExport), and
%  KPI-Last.  
%
%%

    fun enc/2.      % symmetric encryption
    fun penc/2.     % public key encryption

    % types of keys:
    % fun dat/0.    % data keys  (data = 0)
    fun imp/0.      % import keys
    fun exp/0.      % export keys
    fun pin/0.      % key derivation keys
    fun kp/0.       % partial key type

    fun kek/0.      

    fun km/0.       % the security module's master key 
    fun pan/0.      % an account number

    fun c/0.        % some value known by the intruder

    fun key/1.      % key(Type) -- a key of type Type generated by the module or guessed.

    fun ukek1/0.   
    fun ukek2/0.   

    fun pdkey/0.    % a Pin Derivation Key

    fun sec/0.

    proverif 'param selFun = Nounifset'.

    % dominating [imp, exp, pin, kp].

%%%%%%%%%%%% Auxiliary Prolog predicates

    type(0).   % data=0
    type(imp).
    type(exp).
    type(pin).
    
%%%%%%%%%%%% THE INITIAL KNOWLEDGE %%%%%%%%

    rule c.
    % rule dat.  % dat=0
    rule imp.
    rule exp.
    rule pin.
    rule kp.

    rule pan.

    rule enc(kek, km+imp).

    rule enc(pdkey, kek+pin).  % the key being transfered
    rule enc(pdkey, km+pin).  %%% not in THEIR model

    %%% Key Generating/Guessing

    rule enc(key(0),   km+0).
    rule enc(key(imp), km+imp).
    rule enc(key(exp), km+exp).
    rule enc(key(pin), km+pin).

    rule enc(ukek1, km+kp+0).
    rule enc(ukek1, km+kp+imp).
    rule enc(ukek1, km+kp+exp).
    rule enc(ukek1, km+kp+pin).
    
    % For IBM Attack:
    rule enc(ukek2, ukek1+exp).

    % ... and some similar keys
    rule enc(ukek2, ukek1+imp).
    rule enc(ukek2, ukek1+pin).
    rule enc(ukek2, ukek1+exp).
 

%%%%%%%%%%%% INTRUDER RULES %%%%%%%%%%%%%%%

    rule  X, K -> enc(X,K).
    rule  enc(X,K), K -> X.

%%%%%%%%%%%% PROTOCOL RULES %%%%%%%%%%%%%%%

%%% Encipher
    rule  X, enc(Key, km+0)  ->  enc(X, Key).

%%% Decipher
    rule  enc(X, Key), enc(Key, km+0)  ->  X.

%%% Key Import
    rule  enc(Key, KEK+Type), Type, enc(KEK, km+imp)  ->  enc(Key, km+Type)
    :- type(Type).

%%% Key Export
    rule  enc(Key, km+Type), Type, enc(KEK, km+exp)  ->  enc(Key, KEK+Type)
    :- type(Type).


%%% Key Part Import

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rule  enc(pdkey, km)  -> sec.
rule  pdkey -> sec.
rule  enc(pan, pdkey) -> sec.
rule  kek -> sec.
rule  ukek1 -> sec.
rule  ukek2 -> sec.
rule  key(_) -> sec.

query sec.

